#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from PyQt4 import QtGui, QtCore
from PyQt4 import QtWebKit
import os
import email
import re
import random
import urllib
import tempfile
import shutil


SOFTINFO = {
    'name' : 'pyEmlReader',
    'author' : 'emptyhua@gmail.com',
    'site' : 'http://bluehua.org',
    'version' : '0.1 beta',
    'license' : '''pyEmlReader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.
'''}

_NOTHTML = re.compile('<html[^>]*>', re.I)
_NOPHTML = re.compile('</html>', re.I)
_NOMETA = re.compile('<meta[^>]*>', re.I)

def debug_out(str):
    if True:
        print str

class EmlParser(object):
    def __init__(self):
        self.tmp_dir = tempfile.gettempdir() + os.sep + SOFTINFO['name'].lower() + '_tmp'
        if ( not os.path.isdir(self.tmp_dir) ):
            os.mkdir(self.tmp_dir)

    @staticmethod
    def decodeGBstr(str):
        try:
            str = str.decode('gb2312')
        except:
            pass
        return str

    @staticmethod
    def decodeMstr(str):
        str = email.Header.decode_header(email.Header.Header(str))[0][0]
        return str

    @staticmethod
    def decodeMhead(head):
        str = email.Header.decode_header(head)[0][0]
        return str

    @staticmethod
    def getEmail(a):
        ems = email.utils.getaddresses(a)
        rt = []
        for em in ems:
            rt.append(EmlParser.decodeMstr(em[0]) + ' &lt;' + em[1] + '&gt;')
        return ''.join(rt)

    def cleanTemp(self):
        for file in os.listdir(self.tmp_dir):
            os.unlink(self.tmp_dir + os.sep + file)
            debug_out('remove tmp file:' + file)

    def parse(self, file):
        rt = {}
        rt['date'] = ''
        rt['from'] = ''
        rt['to'] = ''
        rt['cc'] = ''
        rt['subject'] = ''
        rt['body'] = ''
        rt['attach'] = []

        f = open(file, 'r')
        debug_out('-' * 30)
        debug_out('parse:' + file)
        msg = email.message_from_file(f)
         
        rt['date'] = msg.get('date')
        debug_out('date:' + rt['date'])
        rt['from'] = EmlParser.getEmail(msg.get_all('from', []))
        debug_out('form:' + rt['from'])
        rt['to'] = EmlParser.getEmail(msg.get_all('to', [])) 
        debug_out('to:' + rt['to'])
        rt['cc'] = EmlParser.getEmail(msg.get_all('cc', [])) 
        debug_out('cc:' + rt['cc'])
        rt['subject'] = EmlParser.decodeMhead(msg.get('subject'))
        debug_out('subject:' + rt['subject'])

        for part in msg.walk():
            if part.is_multipart():
                continue
            debug_out('content type:' + part.get_content_type())
            name = EmlParser.decodeMstr(part.get_param('name'))
            if name :
                name = str.lower(name)
                debug_out('attach:' + name)
                finfo = []
                finfo.append(name)
                fname = random.choice( 'abcdef' ) +  '_' + name
                fname = self.tmp_dir + os.sep + fname 
                finfo.append(fname) 
                rt['attach'].append(finfo)
                f = open(fname , 'wb')
                f.write( part.get_payload( decode=True ) )
                f.close()
                debug_out('attach tmp:' + fname)

            else:
                type = part.get_content_type()
                if ( (type == 'text/html' or type == 'text/plain') and rt['body'] == '' ):
                    body = part.get_payload(decode=True)
                    body = _NOTHTML.sub('', body)
                    body = _NOPHTML.sub('', body)
                    body = _NOMETA.sub('', body)
                    if ( type == 'text/plain' ):
                        body = '<pre>' + body + '</pre>'
                    rt['body'] = body
        debug_out('-' * 30)
        return rt
                

class EmlView(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        
        layout = QtGui.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        self.webView = QtWebKit.QWebView()
        self.webView.setGeometry(QtCore.QRect(0, 0, 1111, 691))
        layout.addWidget(self.webView)
        self.webView.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateAllLinks)
        self.connect(self.webView.page(), QtCore.SIGNAL("linkClicked(const QUrl&)"), self.handleLinkClicked)

        self.webView.setAutoFillBackground(True)

        self.setLayout( layout )
        self.parser = EmlParser()

    def renderEml(self, msg):
        for mk in ['date', 'from', 'to', 'cc', 'subject', 'body']:
            msg[mk] = EmlParser.decodeGBstr(msg[mk])
            
        html = ['<html>']
        html.append('<meta http-equiv="Content-Type" content="text/html; charset="UTF-8" />')
        html.append('<p style="margin-left:20px">')
        html.append('Subject:' + msg['subject'] + '</br>')
        html.append('From:' + msg['from'] + '</br>')
        html.append('To:' + msg['to'] + '</br>')
        html.append('Cc:' + msg['cc'] + '</br>')
        html.append('Date:' + msg['date'] + '</br>')
        html.append('</p>')
        html.append('<hr>')
        ats = msg['attach']
        if ( len(ats) > 0 ):
            html.append('<p style="margin-left:20px">')
            html.append('Attachments:<br />')
            for at in ats:
                html.append('<a href="download://' + urllib.quote_plus(at[1]) + '">' + at[0] + '</a><br />')
            html.append('</p>')
            html.append('<hr>')
        html.append(msg['body'])
        html.append('</html>')
        try:
            str = ''.join(html)
        except:
            str = '<html>email decode fail</html>'
        self.webView.setHtml(str)

         
    def openEml(self, filename):
        if filename == '':
            return
        if not os.path.isfile(filename):
            debug_out('file do not exist')
            QtGui.QMessageBox.warning(self, SOFTINFO['name'], 'file do not exist')
            return
        if not filename.endswith('.eml'):
            debug_out('.eml required')
            QtGui.QMessageBox.warning(self, SOFTINFO['name'], 'you must select a eml file')
            return
        msg = self.parser.parse(filename)
        self.renderEml(msg)

    def handleLinkClicked(self, url):
        uurl = unicode(url.toString())
        debug_out('click url:' + uurl)
        if uurl.startswith('download://'):
            file = uurl.replace('download://', '')
            if ( not os.path.isfile(file) ):
                debug_out(file + ' does not extist')
                return
            st = QtGui.QFileDialog.getSaveFileName(self, 'Save File To', os.path.basename(file))
            st = unicode(st)
            if ( st == '' ):
                return
            shutil.copy(file, st)
            QtGui.QMessageBox.information(self, SOFTINFO['name'], 'Attatchment has been saved')
        else:
            QtGui.QDesktopServices.openUrl(url)


class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)

        self.setWindowTitle(SOFTINFO['name'])

        exit = QtGui.QAction(QtGui.QIcon('icons/exit.png'), 'Exit', self)
        exit.setShortcut('Ctrl+Q')
        exit.setStatusTip('Exit application')
        self.connect(exit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))

        open = QtGui.QAction(QtGui.QIcon('icons/open.png'), 'Open', self)
        open.setShortcut('Ctrl+O')
        open.setStatusTip('Open a eml file')
        self.connect(open, QtCore.SIGNAL('triggered()'), self.selectFileDialog)

        site = QtGui.QAction(QtGui.QIcon('icons/site.png'), 'Help Online', self)
        site.setStatusTip('Visit the author')
        self.connect(site, QtCore.SIGNAL('triggered()'), self.helpOnline)

        about = QtGui.QAction(QtGui.QIcon('icons/about.png'), 'About', self)
        self.connect(about, QtCore.SIGNAL('triggered()'), self.aboutDialog)

        menubar = self.menuBar()
        file = menubar.addMenu('&File')
        file.addAction(open)
        file.addAction(exit)
        
        help = menubar.addMenu('&Help')
        help.addAction(site)
        help.addAction(about)

        self.statusBar()
        self.resize(1024, 768)

        self.webKit = None

    def selectFileDialog(self):
        filename = QtGui.QFileDialog.getOpenFileName(self, 'Select a eml file')
        filename = unicode(filename)
        self.openNewEml(filename)


    def openNewEml(self, filename):
        if self.webKit is None:
            self.webKit = EmlView()
            self.setCentralWidget(self.webKit)
        self.webKit.openEml(filename)

    def helpOnline(self):
        QtGui.QDesktopServices.openUrl(QtCore.QUrl(SOFTINFO['site']))

    def aboutDialog(self):
        msg = [SOFTINFO['name'] + ' ' + SOFTINFO['version'] + '\n']
        msg.append('Author : ' + SOFTINFO['author'] + '\n')
        msg.append('Site : ' + SOFTINFO['site'] + '\n')
        msg.append('License:\n')
        msg.append(SOFTINFO['license'])
        QtGui.QMessageBox.about(self, 'About ' + SOFTINFO['name'], ''.join(msg))
    def closeEvent(self, event):
        if self.webKit is not None:
            self.webKit.parser.cleanTemp();

def main():
    app = QtGui.QApplication(sys.argv)
    main = MainWindow()
    main.show()
    if len(sys.argv) > 1:
        file = sys.argv[1]
        if os.path.isfile(file):
            main.openNewEml(file)
    sys.exit(app.exec_())
    
if __name__ == '__main__':
    main()


